import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
from genetorch import reader
from collections import Counter

def search(df, col, kw):
    return df[col] == kw


def find(a):
    #  合并df
    reader.get_impact(a)
    co_data = pd.DataFrame()
    for i in range(len(a.taglist)):
        co_data = pd.concat([co_data, a.taglist[i]])
        a.co_data = co_data
    a.result = get(co_data)
    print(a.result)
    return get(co_data)


# get为核心分析函数
def get(co_data):
    co_data = co_data.drop_duplicates()
    genelist = list(set(co_data['gene'].to_list()))
    line = []
    for gene in genelist:
        temp_df = co_data[co_data['gene'] == gene]
        sample = list(set(temp_df['tag'].to_list()))
        size = len(sample)
        var = dict(Counter(temp_df['protein'].to_list()))
        var_num = len(var)
        line.append([sample, size, gene, var, var_num])
    result = pd.DataFrame(line, columns=['sample', 'size', 'gene', 'variation', 'variation_number'])
    result = result[~result['variation'].isin([[]])]
    result = result.sort_values(by=['size'], ascending=False)
    result = result.reset_index(drop=True)
    return result


def filter(a, lengthlimit=0.6, rid=['ttn-1', 'cla-1']):
    reader.get_impact(a)
    conc = pd.DataFrame()
    for i in range(len(a.taglist)):
        conc = pd.concat([conc, a.taglist[i]])
    filter = conc.groupby(['gene', 'protein', 'ID', 'base']).filter(lambda x: len(x) <= lengthlimit)
    a.co_data = filter
    result = get(filter)
    result = result[~result['gene'].isin(rid)]
    n = result.shape[0]
    result.index = list(range(n))
    a.result = result
    return result


def plot(result, size=800, c='Reds', title='variation_number', sub_c='RdPu', intersection_factor=0.03):
    list_var = []
    samplesize = 1.4 * max(result['variation_number'])
    m = result
    result2 = result[0:size]
    y = list(result2['size'])
    # 预设图像各种信息
    large = 22
    med = 16
    small = 12
    fig, ax = plt.subplots()
    xlist = list(result2['gene'])
    ylist = list(result2['variation_number'])
    slist = list(500 * result2['variation_number'] / max(result2['variation_number']))
    clist = list(0.7 * result2['variation_number'] / result2['size'])
    vlist = list(result2['variation'])
    samlist = list(result2['sample'])
    for i in range(len(vlist)):
        list_var = str(vlist[i]).split(',')
        if len(list_var) / 2 < 20:
            for j in range(0, len(list_var), 3):
                if j % 2 != 0:
                    list_var[j] = str(list_var[j]) + '\n'
            vlist[i] = ','.join(list_var)
        else:
            vlist[i] = 'variance number =' + str(len(list_var) / 2)

    for i in range(len(samlist)):
        list_sam = str(samlist[i]).split(',')
        if len(list_sam) < 20:
            for j in range(2, len(list_sam), 3):
                list_sam[j] = str(list_sam[j]) + '\n'
            samlist[i] = ','.join(list_sam)
        else:
            samlist[i] = 'sample size =' + str(len(list_sam))
    tlist2 = list(map(str, vlist))
    tlist1 = list(map(str, xlist))
    tlist3 = list(map(str, samlist))
    sc = plt.scatter(x=xlist, y=ylist,
                     s=slist,
                     alpha=0.7,
                     c=plt.get_cmap(c)(clist))
    print(sc.get_offsets()[1][0])
    annot = ax.annotate("", xy=(0, 0), xytext=(10, 10), textcoords="offset points",
                        bbox=dict(boxstyle="round", fc="w"),
                        arrowprops=dict(arrowstyle="-",
                                        color="0.5",
                                        shrinkB=5,
                                        ),
                        fontsize=7)
    annot.set_visible(False)

    def update_annot(ind):

        pos = sc.get_offsets()[ind['ind'][0]]
        annot.xy = pos
        text = "{},\n{},\n{}".format("".join([tlist1[n] for n in ind["ind"]]),
                                     "".join([tlist2[n] for n in ind["ind"]]),
                                     "".join([tlist3[n] for n in ind["ind"]]))
        annot.set_text(text)
        # annot.get_bbox_patch().set_facecolor(cmap(norm(c[ind["ind"][0]])))
        annot.get_bbox_patch().set_alpha(0.4)

    def hover(event):
        vis = annot.get_visible()
        if event.inaxes == ax:
            cont, ind = sc.contains(event)
            if cont:
                update_annot(ind)
                annot.set_visible(True)
                fig.canvas.draw_idle()
            else:
                if vis:
                    annot.set_visible(False)
                    fig.canvas.draw_idle()

    # mouse event to get intersection
    def getintersection(event, color=sub_c, num=intersection_factor):
        if event.inaxes == ax:
            cont, ind = sc.contains(event)
            if cont:
                genename = str([tlist1[n] for n in ind["ind"]][0])
                print(ind['ind'])
                print([tlist1[n] for n in ind["ind"]])
                a = gsamplename(m, [tlist1[n] for n in ind["ind"]][0])
                print(a)
                b = intersect(m, a, i_num=num)
                print(b)
                plot(b, title=title + '---' + genename, c=color)

    fig.canvas.mpl_connect('button_press_event', getintersection)
    fig.canvas.mpl_connect("motion_notify_event", hover)
    # 装饰图像
    plt.gca().set(xlabel='mutation_frequency', ylabel='variation_number')
    plt.xticks([])
    plt.ylim(0, samplesize)
    plt.yticks(fontsize=12)
    plt.title(title, fontsize=12)
    plt.show()


def gsamplename(result, genename):
    if genename in list(result['gene']):
        ind = result[result['gene'].isin([genename])]
        return list(ind['sample'])


def intersect(result, samplename, i_num):
    result.reset_index()
    print(result)
    sn = list(samplename[0])
    sm = list(result['sample'])
    indlist = []
    for i in range(len(sm)):
        tmp = [a for a in sn if a in list(sm[i])]
        if len(tmp) > int(i_num * len(sn)):
            indlist.append(i)
    result = pd.DataFrame(result).drop(index=indlist, axis=0)
    n = result.shape[0]
    result.index = list(range(n))
    return result


def re(n):  # 阶乘
    x = 1
    if n == 0:
        return 0
    elif n == 1:
        return 1
    else:
        for i in range(1, n + 1):
            x *= i
        return x


def comb(m, n):
    if n != 0 and m - n != 0:
        b = re(m) / (re(n) * re(m - n))
    if n == 0 or m - n == 0:
        b = 1
    return b


def give_p(snp_sup, snp_all, num_gene1, num_gene2, num_geneall, inter):
    if num_gene1 >= num_gene2:
        p1 = num_gene1 / num_geneall
        p2 = num_gene2 / num_geneall
    else:
        p1 = num_gene2 / num_geneall
        p2 = num_gene1 / num_geneall
    c_num = comb(num_geneall, inter)
    pdouble = p1 * p2
    p = c_num * (pdouble ** inter) * ((1 - pdouble) ** (num_geneall - inter)) * (
            (snp_all - snp_sup) / (snp_all - num_gene1))
    return p


def inters(list1, list2):
    tmp = [a for a in list1 if a in list2]
    return len(tmp)


def p_val(total, casea, caseb, n, snp):
    p = comb(casea, n) * comb(total - casea, caseb - n) / comb(total, caseb) * (snp - total) / (snp - casea)
    return p


def get_p_all(file, path):
    a = len(file.taglist)
    b = file.co_data.shape[0]
    m = []
    n = []
    sup_gene1 = list(file.result['sample'])
    sup_gene2 = list(file.result['sample'])
    inter = 0
    for i in range(len((sup_gene1))):
        for j in range(len(sup_gene2)):
            inter = inters(sup_gene1[i], sup_gene2[j])
            p = give_p(a, b, len(sup_gene1[i]), len(sup_gene2[j]), a, inter=inter)
            m.append(p)
        n.append(m)
    q = np.asarray(m).reshape(len(sup_gene1), len(sup_gene2))
    m = pd.DataFrame(q, index=list(file.result['gene']), columns=list(file.result['gene']))
    m.to_csv(path)


def get_p_sup(gene, file):
    a = len(file.taglist)
    b = file.co_data.shape[0]
    m = []
    n = []
    sup_gene = list(file.result['sample'])
    namelist = list(file.result['gene'])
    num = namelist.index(gene)
    inter = 0
    for i in range(len((sup_gene))):
        inter = inters(sup_gene[i], sup_gene[num])
        p = give_p(a, b, len(sup_gene[i]), len(sup_gene[num]), a, inter=inter)
        m.append(p)
    n.append(m)
    q = np.asarray(m).reshape(len(sup_gene), 1)
    m = pd.DataFrame(q, index=list(file.result['gene']), columns=['p_val'])
    return m


def merge(a, gene1, gene2):
    ga = a.result[a.result['gene'] == gene1]['sample'].tolist()[0]
    gb = a.result[a.result['gene'] == gene2]['sample'].tolist()[0]
    a.result = a.result[~a.result['gene'].isin([gene1, gene2])]
    lista = ga + gb
    lista = list(set(lista))
    size = len(lista)
    i = a.result.shape[0]
    a.result.loc[i + 2] = [lista, size, gene1 + '+' + gene2, '?', '?']
    print(a.result)
    return a.result


def split(a, candidate_list):
    candi = []
    no_candi = []
    for i in a.taglist:
        if list(i['tag'])[1] in candidate_list:
            candi.append(i)
        else:
            no_candi.append(i)
    return candi, no_candi


def candidate_list(a, gene_name):
    candidate = []
    for i in gene_name:
        candidate.extend(a.result[a.result['gene'] == i]['sample'].tolist()[0])
    candidate = list(set(candidate))
    return candidate


def important_gene(p_frame):
    return p_frame[p_frame['p_val'] < 0.05].index.tolist()


def classification(a, gene):
    p = get_p_sup(gene, a)
    lista = important_gene(p)
    candidate = candidate_list(a, lista)
    m = reader.readfile()
    n = reader.readfile()
    m.taglist, n.taglist = split(a, candidate)
    filter(m, 0.5)
    filter(n, 0.5)
    print(p)
    return m, n, p


def get_p_between(a, genea, geneb):
    if genea != geneb:
        sa = a.result[a.result['gene'] == genea]['sample'].tolist()[0]
        sb = a.result[a.result['gene'] == geneb]['sample'].tolist()[0]
        inter = inters(sa, sb)
        sup = len(a.taglist)
        snp = a.co_data.shape[0]
        p = give_p(sup, snp, len(sa), len(sb), sup, inter=inter)
    else:
        p = 1
    return p


def suppressor_group_p(a, genea):
    sa = a.result[a.result['gene'] == genea]['sample'].tolist()[0]
    sb = a.suppressor_group
    inter = inters(sa, sb)
    sup = len(a.taglist)
    snp = a.co_data.shape[0]
    p = p_val(len(a.taglist), len(sb), len(sa), inter, len(a.co_data))
    return p


def intersect_pro(a, genename):
    c = a
    c.result = c.result.reset_index(drop=True)
    lista = c.result['sample'].tolist()
    listb = c.result[c.result['gene'] == genename]['sample'].tolist()[0]
    c.candidate.append(genename)
    c.candidate = list(set(c.candidate))
    c.suppressor_group.extend(listb)
    c.suppressor_group = list(set(c.suppressor_group))
    print(c.candidate)
    print(len(c.suppressor_group))
    rid = []
    gene = []
    b = reader.readfile()
    for i in range(len(lista)):
        tmp = inters(lista[i], c.suppressor_group)
        if tmp != 0:
            p = get_p_between(c, genename, c.result['gene'].tolist()[i])
            if p > 0.05:
                rid.append(i)
                gene.append(c.result['gene'].tolist()[i])
    b.result = pd.DataFrame(c.result).drop(index=rid, axis=0)
    b.result = b.result[~b.result['gene'].isin(c.candidate)]
    b.taglist = c.taglist
    b.co_data = c.co_data[~c.co_data['gene'].isin(gene)]
    c = a
    return b, c


def plotpro(a, size=800, c='Reds', title='variation_number', sub_c='RdPu', origin=reader.readfile()):
    if len(origin.candidate) == 0:
        origin = a
    samplesize = 1.4 * max(a.result['variation_number'])
    result2 = a.result[0:size]
    fig, ax = plt.subplots()
    xlist = list(result2['gene'])
    ylist = list(result2['variation_number'])
    slist = list(500 * result2['variation_number'] / max(result2['variation_number']))
    clist = list(0.7 * result2['variation_number'] / result2['size'])
    vlist = list(result2['variation'])
    samlist = list(result2['sample'])
    for i in range(len(vlist)):
        list_var = str(vlist[i]).split(',')
        if len(list_var) / 2 < 20:
            for j in range(0, len(list_var), 3):
                if j % 2 != 0:
                    list_var[j] = str(list_var[j]) + '\n'
            vlist[i] = ','.join(list_var)
        else:
            vlist[i] = 'variance number =' + str(len(list_var) / 2)

    for i in range(len(samlist)):
        list_sam = str(samlist[i]).split(',')
        if len(list_sam) < 20:
            for j in range(2, len(list_sam), 3):
                list_sam[j] = str(list_sam[j]) + '\n'
            samlist[i] = ','.join(list_sam)
        else:
            samlist[i] = 'sample size =' + str(len(list_sam))
    tlist2 = list(map(str, vlist))
    tlist1 = list(map(str, xlist))
    tlist3 = list(map(str, samlist))
    sc = plt.scatter(x=xlist, y=ylist,
                     s=slist,
                     alpha=0.7,
                     c=plt.get_cmap(c)(clist))
    print(sc.get_offsets()[1][0])
    annot = ax.annotate("", xy=(0, 0), xytext=(10, 10), textcoords="offset points",
                        bbox=dict(boxstyle="round", fc="w"),
                        arrowprops=dict(arrowstyle="-",
                                        color="0.5",
                                        shrinkB=5,
                                        ),
                        fontsize=7)
    annot.set_visible(False)

    def update_annot(ind):

        pos = sc.get_offsets()[ind['ind'][0]]
        annot.xy = pos
        text = "{},\n{},\n{}".format("".join([tlist1[n] for n in ind["ind"]]),
                                     "".join([tlist2[n] for n in ind["ind"]]),
                                     "".join([tlist3[n] for n in ind["ind"]]))
        annot.set_text(text)
        # annot.get_bbox_patch().set_facecolor(cmap(norm(c[ind["ind"][0]])))
        annot.get_bbox_patch().set_alpha(0.4)

    def hover(event):
        vis = annot.get_visible()
        if event.inaxes == ax:
            cont, ind = sc.contains(event)
            if cont:
                update_annot(ind)
                annot.set_visible(True)
                fig.canvas.draw_idle()
            else:
                if vis:
                    annot.set_visible(False)
                    fig.canvas.draw_idle()

    # mouse event to get intersection
    def getintersection(event, color=sub_c):
        if event.inaxes == ax:
            cont, ind = sc.contains(event)
            if cont:
                genename = str([tlist1[n] for n in ind["ind"]][0])
                print(ind['ind'])
                b, c = intersect_pro(origin, genename)
                plotpro(b, title=title + '---' + genename, c=color, origin=c)

    fig.canvas.mpl_connect('button_press_event', getintersection)
    fig.canvas.mpl_connect("motion_notify_event", hover)
    # 装饰图像
    plt.gca().set(xlabel='mutation_frequency', ylabel='variation_number')
    plt.xticks([])
    plt.ylim(0, samplesize)
    plt.yticks(fontsize=12)
    plt.title(title, fontsize=12)
    plt.show()



import pandas as pd


def add_candidate(readfile, gene):
    ge = readfile.candidate
    co = readfile.co_data
    data = readfile.result
    name = data[data['gene'] == gene]['sample'].to_list()[0]
    for j in name:
        df1 = co[co['tag'] == j]
        df2 = df1[df1['gene'] == gene]['protein'].to_list()[0]
        ge[j].append([gene,df2])
    readfile.candidate = ge


def del_candidate(readfile, gene):
    ge = readfile.candidate
    for k, v in ge.items():
        if gene in v:
            m = list(filter((1).__ne__, v))
            ge[k] = m
    readfile.candidate = ge

def genelist_out(readfile,path):
    ge = readfile.candidate
    ge = pd.DataFrame.from_dict(ge,orient='index')
    ge.to_csv(path)

def genelist_cls(readfile):
    readfile.candidate = {}
    for i in readfile.names:
        readfile.candidate[i] = []



import pandas as pd
import io
import os
import shutil
import time


def search(df, col, kw):
    return df[col] == kw


class pool:
    def __init__(self, path=None):
        self.path = path
        self.taglist = []
        self.names = []
        self.readfile()
        self.co_data = pd.DataFrame()
        self.result = pd.DataFrame()
        self.suppressor_group = []
        self.candidate = []
        self.list = []

    def readfile(self):
        time1 = time.time()
        if self.path != None:
            files = os.listdir(self.path)
            simp_list = []
            for file in files:
                if not os.path.isdir(file):
                    names = os.path.basename(file).split('.')
                    if names[-1] == 'vcf':
                        f = read_vcf(self.path + '/' + file)
                        tag = names[0]
                        simp_list.append(simp_file(f,tag))
            self.taglist = simp_list
            time4 = time.time()
            print(time4-time1)


class getpool:
    def __init__(self, filepath, filenames):
        self.filepath = filepath
        self.filenames = filenames
        self.temp = filepath + '\\temp'
        self.taglist = []
        self.names = []
        self.splitfile()
        self.suppressor_group = []
        self.candidate = []
        self.list = []

    def splitfile(self):
        os.mkdir(self.temp)
        files = []
        a = os.listdir(self.filepath)
        for i in range(len(a)):
            if os.path.isdir(self.filepath + '/' + a[i]):
                files.append(a[i])

        self.names = []
        for i in range(len(files)):
            self.names.append(files[i].split('_')[0])

        files2 = []
        for i in range(len(files)):
            files2.append(os.listdir(self.filepath + '/' + files[i]))
            for j in range(len(files2[i])):
                if files2[i][j] == self.filenames:
                    shutil.copy(self.filepath + '/' + files[i] + '/' + self.filenames,
                                self.temp + '/' + self.names[i] + '.vcf')

        files3 = os.listdir(self.temp)
        filelist = []
        filename = []

        for file in files3:
            if not os.path.isdir(file):
                f = read_vcf(self.temp + '/' + file)
                filelist.append(f)
                filename.append(os.path.basename(file).split('.')[0])

        simp_list = []
        for i in range(len(filelist)):
            simp_list.append(simp_file(filelist[i]))
        self.taglist = simp_list
        for i in range(len(self.taglist)):
            self.taglist[i].insert(loc=len(self.taglist[i].columns), column='tag', value=self.names[i])


def get_impact(a):
    newtaglist = []
    for i in range(len(a.taglist)):
        n = a.taglist[i]
        lista = []
        listb = []
        splice_donor = n[n['type'] == 'splice_donor_variant&intron_variant']['protein'].tolist()
        splice_acceptor = n[n['type'] == 'splice_acceptor_variant&intron_variant']['protein'].tolist()
        for i in range(len(splice_donor)):
            lista.append('X_donor')
        for i in range(len(splice_acceptor)):
            listb.append('X_acceptor')
        c = pd.DataFrame(n[n['type'] == 'splice_donor_variant&intron_variant'])
        d = pd.DataFrame(n[n['type'] == 'splice_acceptor_variant&intron_variant'])
        c['protein'] = lista
        d['protein'] = listb
        n = pd.concat([n, c, d])
        indp = search(n, 'protein', '')
        indq = search(n, 'protein', 'nan')
        ind = search(n, 'type', 'synonymous_variant')
        m = n.loc[ind, :]
        b = n.loc[indp, :]
        e = n.loc[indq, :]
        n = pd.concat([m, n, e]).drop_duplicates(keep=False)
        total = pd.concat([n, b]).drop_duplicates(keep=False)
        newtaglist.append(total)
    a.taglist = newtaglist
    return a.taglist


# read info
def simp_file(raw_df,tag):
    b = []
    info = raw_df['INFO'].to_list()
    for i in info:
        lst = i.split('|')
        b.append([lst[3],lst[1],lst[10],tag])
    genelist = pd.DataFrame(b, columns=['gene', 'type', 'protein','tag'])
    return genelist


# read vcf from file using pandas dataframe
def read_vcf(path_a):
    with open(path_a, 'r') as f:
        lines = [l for l in f if not l.startswith('##')]
    return pd.read_csv(
        io.StringIO(''.join(lines)),
        dtype={'#CHROM': str, 'POS': int, 'ID': str, 'REF': str, 'ALT': str,
               'QUAL': str, 'FILTER': str, 'INFO': str},
        sep='\t'
    ).rename(columns={'#CHROM': 'CHROM'})



import pandas as pd
import numpy as np
import random
from genetorch import finder


def creat_list(h, n):
    return random.sample(range(1, h + 1), n)


def fake_data(a, gene_name):
    dist = list(a.result[a.result['gene'] != gene_name]['size'])
    h = len(a.taglist)
    faker = []
    for i in dist:
        faker.append(list(creat_list(h, i)))
    return faker


def inters(list1, list2):
    tmp = [a for a in list1 if a in list2]
    return len(tmp)


def give_p(snp_sup, snp_all, num_gene1, num_gene2, num_geneall, inter):
    if num_gene1 >= num_gene2:
        p1 = num_gene1 / num_geneall
        p2 = num_gene2 / num_geneall
    else:
        p1 = num_gene2 / num_geneall
        p2 = num_gene1 / num_geneall
    c_num = comb(num_geneall, inter)
    pdouble = p1 * p2
    p = c_num * (pdouble ** inter) * ((1 - pdouble) ** (num_geneall - inter)) * (
            (snp_all - snp_sup) / (snp_all - num_gene1))
    return p


def comb(m, n):
    if n != 0 and m - n != 0:
        b = re(m) / (re(n) * re(m - n))
    if n == 0 or m - n == 0:
        b = 1
    return b


def re(n):  # 阶乘
    x = 1
    if n == 0:
        return 0
    elif n == 1:
        return 1
    else:
        for i in range(1, n + 1):
            x *= i
        return x


def test(snp, sup, dist, probe, threshold):
    p2 = (snp - sup) / (snp - len(probe))
    sup_gene = dist
    m = []
    for i in range(len((sup_gene))):
        inter = inters(sup_gene[i], probe)
        p = finder.p_val(sup, len(sup_gene[i]), len(probe), inter, snp) * p2
        if p <= threshold:
            m.append(p)
    return len(m)


def false_positive(a, gene_name, threshold=[0.05, 0.04, 0.03, 0.02, 0.01]):
    snp = len(a.co_data)
    sup = len(a.taglist)
    probelen = int(a.result[a.result['gene'] == gene_name]['size'])

    m = []
    for i in range(1000):
        probe = list(random.sample(range(1, sup), probelen))
        faked = fake_data(a, gene_name)
        line = []
        for i in threshold:
            p = test(snp, sup, faked, probe, i)
            line.append(p)
            m.append(line)
    print(m)
    mat = np.asarray(m)
    mat = mat.transpose()
    pre = pd.DataFrame(index=threshold, data=mat)
    pre['average'] = pre.mean(axis=1)
    print(pre['average'])
    return pre



import pandas as pd
import scipy.cluster.hierarchy as hct


class AminoAcid:
    def __init__(self, pro, num):
        self.data = pro[pro['num'] == str(num)]
        self.num = num
        self.name = self.data['aa'].to_list()[0]
        self.x = self.data['x'].to_list()
        self.y = self.data['y'].to_list()
        self.z = self.data['z'].to_list()
        self.x = [float(n) for n in self.x]
        self.y = [float(n) for n in self.y]
        self.z = [float(n) for n in self.z]
        self.median = self.get_grav()

    def get_grav(self):
        n = len(self.x)
        return [sum(self.x) / n, sum(self.y) / n, sum(self.z) / n]


def mat(aa_list):
    matrix = [n.median for n in aa_list]
    ind = [n.name + str(n.num) for n in aa_list]
    return pd.DataFrame(matrix, index=ind)


def cluster_plot(matrix):
    link = hct.linkage(matrix, method='complete', metric='euclidean')
    hct.dendrogram(link, leaf_font_size=10, labels=matrix.index)


def get_num(result, target):
    data = result[result['gene'] == target]['variation'].to_list()[0]
    num_lst = []
    keys = list(data.keys())
    for i in keys:
        if i[-1] != '*':
            num_lst.append("".join(list(filter(str.isdigit, i))))
    while '' in num_lst:
        num_lst.remove('')
    num_lst = [int(n) for n in num_lst]
    return list(set(num_lst))


def cluster(model_path, result, gene):
    model = []
    with open(model_path) as f:
        for i in f:
            if i.split(' ')[0] == 'ATOM':
                line = i.split(' ')
                while '' in line:
                    line.remove('')
                while '\n' in line:
                    line.remove('\n')
                model.append([line[3], line[-7].strip('A'), line[-6], line[-5], line[-4]])

    model = pd.DataFrame(model,
                         columns=['aa', 'num', 'x', 'y', 'z'])
    max_num = int(model['num'].to_list()[-1])
    num_lst = get_num(result, gene)
    num_lst = [str(n) for n in num_lst if n < max_num]
    aa_lst = []
    for i in num_lst:
        aa_lst.append(AminoAcid(model, i))
    matrix = mat(aa_lst)
    cluster_plot(matrix)




# to compare date with mutiple aim
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from Gangler import prepare
from matplotlib import pyplot
from collections import Counter
import time


def initiation(a):
    gene = a.result['gene'].tolist()
    variation_num = a.result['variation_number'].tolist()
    # normalization
    # n = a.result[a.result['gene'] == 'ttn-1']['variation_number'].tolist()[0]
    n = sum(variation_num)
    variation_num = [a * 100 / n for a in variation_num]
    pool_file = pd.DataFrame({'gene': gene, 'variation_number': variation_num})
    return pool_file


class snpool:
    def __init__(self, readfile_list, target_list):
        self.target = target_list
        self.initiation_list = []
        self.result = []
        self.readfile_list = readfile_list
        self.data = []
        time1 = time.time()
        for i in self.readfile_list:
            self.data.append(gtfilter(i, 4, rid=[]))
        time2 = time.time()
        print('seperate data' + str(time2-time1))
        self.variation_list = pd.DataFrame()
        self.working()
        time3 = time.time()
        print('get result' + str(time3 - time2))

    def ext_gene(self):
        genelist = []
        for i in self.data:
            genelist.extend(i['gene'].to_list())
        return list(set(genelist))

    def working(self):
        genelist = self.ext_gene()
        data = []
        for i in self.data:
            local_gene = i['gene'].to_list()
            local_var = i['variation_number'].to_list()
            local_tab = dict(zip(local_gene, local_var))
            for j in genelist:
                if not j in local_gene:
                    local_tab[j] = 0
            sum_var = sum(local_tab.values())
            for k, v in local_tab.items():
                local_tab[k] = (v + 0.1) * 100 / sum_var
            data.append(local_tab)
        sum_dict = {}
        for _ in data:
            for k, v in _.items():
                sum_dict.setdefault(k, []).append(v)
        cal_dict = {}
        for k, v in sum_dict.items():
            cal_dict[k] = stat(v)
        self.result = pd.DataFrame.from_dict(cal_dict, orient='index', columns=self.target)
        self.variation_list = pd.DataFrame.from_dict(sum_dict, orient='index', columns=self.target)

    def plot(self, gene):
        x = self.target
        y = list(self.variation_list.loc[gene].values)
        plt.scatter(x, y)
        plt.title('Frequency of ' + gene)


# pick gene and variation_number

def stat(list_ele):
    total = 0
    iter_num = len(list_ele)
    new_list = []
    j = 1
    for i in range(iter_num):
        total = total + list_ele[i]
    for i in range(iter_num):
        if total != 0:
            m = list_ele[i] / total
        else:
            m = 1
        j = j * m
        new_list.append(m)
    alt = []
    for i in list_ele:
        alt.append(i)
    alt.sort()
    sec = alt[-2]
    if max(list_ele) != min(list_ele):
        j = (j / (max(new_list) * (max(list_ele) - sec) ** (len(list_ele) - 1))) ** (1 / (len(list_ele) - 1))
    else:
        j = 'NA'
    m_list = ['_'] * len(new_list)
    if j != 'NA':
        for i in range(len(new_list)):
            if new_list[i] == max(new_list):
                m_list[i] = j / new_list[i]
    return m_list


def get(co_data):
    co_data = co_data.drop_duplicates()
    genelist = list(set(co_data['gene'].to_list()))
    line = []
    for gene in genelist:
        temp_df = co_data[co_data['gene'] == gene]
        sample = list(set(temp_df['tag'].to_list()))
        size = len(sample)
        var = dict(Counter(temp_df['protein'].to_list()))
        var_num = len(var)
        line.append([sample, size, gene, var, var_num])
    result = pd.DataFrame(line, columns=['sample', 'size', 'gene', 'variation', 'variation_number'])
    result = result[~result['variation'].isin([[]])]
    result = result.sort_values(by=['size'], ascending=False)
    result = result.reset_index(drop=True)
    return result


def gtfilter(a, lengthlimit=4, rid=[]):
    prepare.get_impact(a)
    a.candidate = []
    a.suppressor_group = []
    conc = pd.DataFrame()
    for i in range(len(a.taglist)):
        conc = pd.concat([conc, a.taglist[i]])
    filter = conc.groupby(['gene', 'protein']).filter(lambda x: len(x) <= lengthlimit)
    a.co_data = filter
    result = get(filter)
    result = result[~result['gene'].isin(rid)]
    n = result.shape[0]
    result.index = list(range(n))
    a.result = result
    return result


def search(df, col, kw):
    return df[col] == kw



from intermine.webservice import Service
import pandas as pd

class Reported_gene:
    def __init__(self,keyword):
        self.keyword = keyword
        self.data = self.query()
        self.gene = []
        for i in self.data:
            self.gene.append(i['symbol'])

    def query(self):
        service = Service("http://im-dev1.wormbase.org/tools/wormmine/service")
        query = service.new_query("Gene")
        query.add_view("primaryIdentifier", "secondaryIdentifier", "symbol")
        query.add_constraint("allele.phenotype.name", "CONTAINS", self.keyword, code="A")
        return query.rows()



import pandas as pd
import io
import os
import shutil


class stockfile:
    def __init__(self, filepath, filenames, outpath):
        self.filepath = filepath
        self.filenames = filenames
        self.temp = filepath + '\\temp'
        self.outpath = outpath
        self.simplist = []
        self.names = []
        self.stock()

    def stock(self):
        os.mkdir(self.temp)
        files = []
        a = os.listdir(self.filepath)
        for i in range(len(a)):
            if os.path.isdir(self.filepath + '/' + a[i]):
                files.append(a[i])

        self.names = []
        for i in range(len(files)):
            self.names.append(files[i].split('_')[0])

        files2 = []
        for i in range(len(files)):
            files2.append(os.listdir(self.filepath + '/' + files[i]))
            for j in range(len(files2[i])):
                if files2[i][j] == self.filenames:
                    shutil.copy(self.filepath + '/' + files[i] + '/' + self.filenames,
                                self.temp + '/' + self.names[i] + '.vcf')

        files3 = os.listdir(self.temp)
        filelist = []
        filename = []

        # read vcf from file using pandas dataframe
        def read_vcf(path_a):
            with open(path_a, 'r') as f:
                lines = [l for l in f if not l.startswith('##')]
            return pd.read_csv(
                io.StringIO(''.join(lines)),
                dtype={'#CHROM': str, 'POS': int, 'ID': str, 'REF': str, 'ALT': str,
                       'QUAL': str, 'FILTER': str, 'INFO': str},
                sep='\t'
            )

        for file in files3:
            if not os.path.isdir(file):
                f = read_vcf(self.temp + '/' + file)
                filelist.append(f)
                filename.append(os.path.basename(file).split('.')[0])

        # read info
        def simp_file(raw_df):
            b = []
            for i, row in raw_df.iterrows():
                b.append(row['INFO'].split('|'))
            c = []
            for i in range(len(b)):
                c.append([b[i][9], b[i][10], b[i][1], b[i][2], b[i][3], b[i][4], b[i][6], '', ''])
            genelist = pd.DataFrame(c, columns=['nucleotide', 'amino acid', 'type', 'impact', 'gene', 'WBGeneID',
                                                'transcript', 'allele', 'comments'])
            return genelist

        simp_list = []
        for i in range(len(filelist)):
            a = simp_file(filelist[i])
            b = filelist[i].drop(['INFO', 'FILTER', 'ID', 'REF', 'ALT', 'QUAL','FORMAT','unknown'], axis=1)
            c = pd.concat([b, a], axis=1, join='outer')
            simp_list.append(c)
        self.simplist = simp_list

        for i in range(len(self.simplist)):
            self.simplist[i].to_csv(self.outpath + '/' + self.names[i] + '.csv', index=False, sep='\t')
        print('Output csv is in ' + self.outpath)



from genetorch import finder
from genetorch import stocker
from genetorch import reader
from genetorch import simulator
from genetorch import genelist
from genetorch import wormbase
from genetorch import pcluster